#version 0.2.0

samples = ['federal_interest_rates_kaggle', \
           'financial_indicators_US_stocks_kaggle', \
           'world_development_indicator_kaggle', \
           'average_interest_rates_US_treasury', \
           'stochastic_random_walk']
